#include "grbl.h"

#ifdef lcd_h
#define lcd_e_delay() _delay_us(LCD_DELAY_ENABLE_PULSE)
#define lcd_e_high() LCD_PORT |= (1<<LCD_E_BIT)
#define lcd_e_low() LCD_PORT &= ~(1<<LCD_E_BIT)
#define lcd_rw_high() LCD_PORT |= (1<<LCD_RW_BIT)
#define lcd_rs_high() LCD_PORT |= (1<<LCD_RS_BIT)
#define lcd_rw_low() LCD_PORT &= ~(1<<LCD_RW_BIT)
#define lcd_rs_low() LCD_PORT &= ~(1<<LCD_RS_BIT)

static void lcd_write(uint8_t data, uint8_t rs) {
    if (rs) {
        lcd_rs_high();
    } else {
        lcd_rs_low();
    }
    lcd_rw_low();

    LCD_DATA_DDR = 0xff;

    LCD_DATA_PORT = data;
    lcd_e_high();
    lcd_e_delay();
    lcd_e_low();
} /* lcd_write */

static uint8_t lcd_read(uint8_t rs) {
    uint8_t data;

    if (rs) {
        lcd_rs_high();
    } else {
        lcd_rs_low();
    }
    lcd_rw_high();

    LCD_DATA_DDR = 0x00;

    lcd_e_high();
    lcd_e_delay();
    data = LCD_DATA_PIN;
    lcd_e_low();

    return(data);
} /* lcd_read */

static void lcd_waitbusy() {
    while(lcd_read(0) & (1<<LCD_BUSY)) {}
    _delay_us(LCD_DELAY_BUSY_FLAG);
} /* lcd_waitbusy */

void lcd_command(uint8_t cmd) {
    lcd_waitbusy();
    lcd_write(cmd,0);
} /* lcd_command */

void lcd_data(uint8_t data) {
    lcd_waitbusy();
    lcd_write(data,1);
} /* lcd_data */

void lcd_gotoxy(uint8_t x, uint8_t y) {
    if (0==y) {
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE1+(x>>1));
    } else if (1==y) {
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE2+(x>>1));
    } else if (2==y) {
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE3+(x>>1));
    } else { /* 3==y */
        lcd_command((1<<LCD_DDRAM)+LCD_START_LINE4+(x>>1));
    }
} /* lcd_gotoxy */

void lcd_putc(char c) {
    lcd_waitbusy();
    lcd_write(c,1);
} /* lcd_putc */

void lcd_puts(const char *s) {
    register char c;
    while ((c = *s++)) {
        lcd_putc(c);
    }
} /* lcd_puts */

void lcd_puts_p(const char *progmem_s) {
    register char c;
    while ((c = pgm_read_byte(progmem_s++))) {
        lcd_putc(c);
    }
} /* lcd_puts_p */

void lcd_init() {
    LCD_DATA_DDR = 0xff;
    LCD_DDR |= ((1<<LCD_RS_BIT)|(1<<LCD_RW_BIT)|(1<<LCD_E_BIT));
    _delay_us(LCD_DELAY_BOOTUP);

    LCD_DATA_PORT = ((1<<4)|(1<<5));

    lcd_e_high();
    lcd_e_delay();
    lcd_e_low();
    _delay_us(LCD_DELAY_INIT);

    lcd_e_high();
    lcd_e_delay();
    lcd_e_low();
    _delay_us(LCD_DELAY_INIT_REP);

    lcd_command(LCD_DISP_OFF);
    lcd_command((1<<LCD_CLR));
    lcd_command(LCD_ENTRY_INC_);
    lcd_command(LCD_DISP_ON);
} /* lcd_init */

void glcd_invert(uint8_t x, uint8_t y, uint8_t len) {
    if (y > 1) {
        x += (8*2);
        y -= 2;
    }
    lcd_command(LCD_FUNCTION_8BIT_EXTENDED);
    for (uint8_t i=0; i<(64/2); ++i) {
        lcd_command((0x80|i));
        lcd_command(0x80);
        for (uint8_t j=0; j<(16*2); ++j) {
            if ((i >= (y*16)) && (i < (y*16 + 16)) && (j >= x) && (j < (x+len))) {
                lcd_data(0xff);
            } else {
                lcd_data(0x00);
            }
        }
    }
    lcd_command(LCD_FUNCTION_8BIT_BASIC);
} /* glcd_invert */
#endif /* lcd_h */
